# ---------------------------------------------------
# HELPER SCRIPT TO INSTALL AND UPDATE RMOSEK
#
# To activate this script, call
# >> source(filepath_of_this_script)
#
# Warning: Moving or copying this file outside the
# mosek distrbution breaks its ability to determine
# the default value of 'PKG_MOSEKHOME'.
# ---------------------------------------------------


available.rmosek <- function(
  repos   = "http://download.mosek.com/R/9.0",
  filters = c(),
  ...)
{
  available.packages(repos=repos, filters=filters, ...)
}


install.rmosek <- function(
  pkgs           = "Rmosek",
  repos          = "http://download.mosek.com/R/9.0",
  MSK_BINDIR     = guess_mosek_bindir_from_scriptpath(),
  MSK_HEADERDIR  = guess_mosek_headerdir_from_scriptpath(),
  MSK_LIB        = "",
  using_pkgbuild = is_windows(),
  using_sysenv   = is_windows(),
  type           = "source",
  INSTALL_opts   = "--no-multiarch",
  configure.vars = ifelse(using_sysenv, "", paste0("MSK_BINDIR=",MSK_BINDIR," ","MSK_HEADERDIR=",MSK_HEADERDIR," ","MSK_LIB=",MSK_LIB)),
  ...)
{
  if( using_sysenv )
  {
    Sys.setenv(MSK_BINDIR=MSK_BINDIR)
    Sys.setenv(MSK_HEADERDIR=MSK_HEADERDIR)
    Sys.setenv(MSK_LIB=MSK_LIB)
  }

  if( !using_pkgbuild )
  {
    install.packages(pkgs=pkgs, repos=repos, type=type, INSTALL_opts=INSTALL_opts, configure.vars=configure.vars, ...)
  }
  else
  {
    if(!require("pkgbuild")) 
    { 
      stop("Package 'pkgbuild' not available. Either install it or add argument 'using_pkgbuild=FALSE' to this function call.")
    }
    pkgbuild::with_build_tools({ 
    install.packages(pkgs=pkgs, repos=repos, type=type, INSTALL_opts=INSTALL_opts, configure.vars=configure.vars, ...)
    }, debug=TRUE)
  }
}


update.rmosek <- function(
  oldPkgs        = "Rmosek",
  repos          = "http://download.mosek.com/R/9.0",
  MSK_BINDIR     = guess_mosek_bindir_from_scriptpath(),
  MSK_HEADERDIR  = guess_mosek_headerdir_from_scriptpath(),
  MSK_LIB        = "",
  using_pkgbuild = is_windows(),
  using_sysenv   = is_windows(),
  type           = "source",
  INSTALL_opts   = "--no-multiarch",
  configure.vars = ifelse(using_sysenv, "", paste0("MSK_BINDIR=",MSK_BINDIR," ","MSK_HEADERDIR=",MSK_HEADERDIR," ","MSK_LIB=",MSK_LIB)),
  ...)
{
  if( using_sysenv ) 
  {
    Sys.setenv(MSK_BINDIR=MSK_BINDIR)
    Sys.setenv(MSK_HEADERDIR=MSK_HEADERDIR)
    Sys.setenv(MSK_LIB=MSK_LIB)
  }

  if( !using_pkgbuild )
  {
    update.packages(oldPkgs=oldPkgs, repos=repos, type=type, INSTALL_opts=INSTALL_opts, configure.vars=configure.vars, ...)
  }
  else
  {
    if(!require("pkgbuild")) 
    { 
      stop("Package 'pkgbuild' not available. Either install it or add argument 'using_pkgbuild=FALSE' to this function call.")
    }
    pkgbuild::with_build_tools({
    update.packages(oldPkgs=oldPkgs, repos=repos, type=type, INSTALL_opts=INSTALL_opts, configure.vars=configure.vars, ...)
    }, debug=TRUE)
  }
}


remove.rmosek <- function(
  pkgs = "Rmosek",
  ...)
{
  remove.packages(pkgs=pkgs, ...)
}


install.pkgbuild <- function(...)
{
  install_github <- source("https://raw.githubusercontent.com/r-lib/remotes/master/install-github.R")$value
  install_github("r-lib/pkgbuild", ...)
}


# eager evaluation of default values
do.call(function(env)
{
  is_windows <- function()
  {
    (.Platform$OS.type=="windows")
  }

  scriptpath <- function()
  {
    path <- getSrcDirectory(function(){})
    if(length(path)==0) {
      path <- tryCatch(dirname(sys.frame(1)$ofile), error=function(e){ getwd() })
    }
    if( file.exists(file.path(path,'setup.R')) ) { path } else { character(0) }
  }

  guess_mosek_bindir_from_scriptpath <- function()
  {
    normalizePath(file.path(scriptpath(),'..','bin'))
  }

  guess_mosek_headerdir_from_scriptpath <- function()
  {
    normalizePath(file.path(scriptpath(),'..','h'))
  }

  for (fn in c('install.rmosek','update.rmosek')) {
    f <- get(fn)
    for (arg in c('MSK_BINDIR','MSK_HEADERDIR','MSK_LIB','using_pkgbuild','using_sysenv')) {
      formals(f)[[arg]] <- eval(formals(f)[[arg]])
    }
    assign(fn,f,pos=env)
  }
}, list(env=environment()))


cat("
Functions loaded:
   available.rmosek   - list all Rmosek packages on repository.
   install.rmosek     - install Rmosek package.
   update.rmosek      - update Rmosek package.
   remove.rmosek      - remove Rmosek package.
   install.pkgbuild   - install pkgbuild; a recommended package on Windows 
                        to resolve Rtools.

The *.rmosek functions wrap the corresponding utils::*.packages functions
with the following default argument values:
   pkgs           = \"", formals(install.rmosek)[['pkgs']] ,"\"
   repos          = \"", formals(install.rmosek)[['repos']], "\"

configuration variables:
   MSK_BINDIR     = \"", formals(install.rmosek)[['MSK_BINDIR']], "\"
                    # if empty, autoconfigured from mosek executable on PATH.

   MSK_HEADERDIR  = \"", formals(install.rmosek)[['MSK_HEADERDIR']], "\"
                    # if empty, autoconfigured from 'MSK_BINDIR'.

   MSK_LIB        = \"", formals(install.rmosek)[['MSK_LIB']], "\"
                    # if empty, autoconfigured from 'MSK_BINDIR'.

and installation styles:
   using_pkgbuild = ", formals(install.rmosek)[['using_pkgbuild']], "
                    # whether to execute in the 'pkgbuild' environment.
                    
   using_sysenv   = ", formals(install.rmosek)[['using_sysenv']], "
                    # whether to transmit configuration variables via Sys.setenv()
                    # as opposed to configure.vars.

See, e.g., `show(install.rmosek)` for its implementation.

", sep="")

